
#ifndef _CONFIG_H_
#define _CONFIG_H_

#define LIBGBSED_PRIVATE    1



/* uncomment this if you know that you want to use perl's malloc
   on a version of perl or architecture where this is not supported */ 
/* #define REALLY_USE_PERL_MALLOC */

#endif /* _CONFIG_H_ */
